@@ -1,6 +1,6 @@
 package JE::Code;
 
-our $VERSION = '0.024';
+our $VERSION = '0.025';
 
 use strict;
 use warnings; no warnings 'utf8';
@@ -13,6 +13,7 @@ our @EXPORT_OK = 'add_line_number';
 
 use constant T => ${AINT}; # perl doesn’t optimise if(${AINT}) away
 
+require JE::Object::Error;
 require JE::Object::Error::ReferenceError;
 require JE::Object::Error::SyntaxError;
 require JE::Object::Error::TypeError;
@@ -140,7 +141,7 @@ sub execute {
 	}
 	else {
 		# Catch-all for any errors not dealt with elsewhere
-		ref $@ eq '' and $@ = new JE::Object::Error::TypeError
+		ref $@ eq '' and $@ = new JE::Object::Error
 			$global, add_line_number $@;
 	}
 
@@ -179,7 +180,7 @@ sub add_line_number {
 
 package JE::Code::Statement; # This does not cover expression statements.
 
-our $VERSION = '0.024';
+our $VERSION = '0.025';
 
 use subs qw'_create_vars _eval_term';
 use List::Util 'first';
@@ -615,7 +616,7 @@ sub _create_vars {  # Process var and function declarations
 
 package JE::Code::Expression;
 
-our $VERSION = '0.024';
+our $VERSION = '0.025';
 
 # B::Deparse showed me how to get these values.
 use constant nan => sin 9**9**9;
@@ -1296,7 +1297,7 @@ sub _eval_term {
 
 package JE::Code::Subscript;
 
-our $VERSION = '0.024';
+our $VERSION = '0.025';
 
 sub str_val {
 	my $val = (my $self = shift)->[1];
@@ -1308,7 +1309,7 @@ sub str_val {
 
 package JE::Code::Arguments;
 
-our $VERSION = '0.024';
+our $VERSION = '0.025';
 
 sub list {
 	my $self = shift;
